//
//  KJSegmentController.swift
//  12-swift分段选择
//
//  Created by apple on 2017/9/7.
//  Copyright © 2017年 HEJJY. All rights reserved.
//

import UIKit

let KSegmentH : CGFloat = 40

class KJSegmentController: UIViewController {
    
    var lastOffsetX : CGFloat = 0
    
    var titleArray : [String] = [String]() {
        
        didSet {
            segmentView.titleArray = titleArray
        }
        
    }
    
    var childControllers : [UIViewController] = [UIViewController]()
    
    fileprivate lazy var segmentView : KJSegmentView = {
    
        let segment = KJSegmentView(frame: CGRect(x: 0, y: 0, width: KScreenW, height: KSegmentH))
        segment.backgroundColor = UIColor.white
        segment.delegate = self
        return segment
        
    }()
    
    fileprivate lazy var scrollView : UIScrollView = {
        
        let scroll = UIScrollView(frame: CGRect(x: 0, y: KSegmentH, width: KScreenW, height: self.view.height - KSegmentH))
        scroll.backgroundColor = UIColor.white
        scroll.isPagingEnabled = true
        scroll.delegate = self
        scroll.bounces = false
        return scroll
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        
        view.backgroundColor = UIColor.yellow
        
        view.addSubview(segmentView)
        view.addSubview(scrollView)
        
        // 把控制器视图添加到scrollView
        addChildControllers()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    

    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

}

// #MARK - 公共方法
extension KJSegmentController {
    
    func addChildControllers() {
        
        scrollView.contentSize = CGSize(width: CGFloat(childControllers.count) * KScreenW, height: scrollView.height)
        
        // 把控制器视图添加到scrollview上
        for i in 0..<childControllers.count {
            
            let controller = childControllers[i]
            
            controller.view.frame = CGRect(x: CGFloat(i) * scrollView.width, y: 0, width: scrollView.width, height: scrollView.height)
            
            self.addChildViewController(controller)
            scrollView.addSubview(controller.view)
        }
    }
    
    func addToParentController(viewController : UIViewController) {
        
        viewController.view.addSubview(self.view)
        viewController.addChildViewController(self)
        
    }
}

extension KJSegmentController : UIScrollViewDelegate {
    
    func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        
        print(#function)
        
        lastOffsetX = scrollView.contentOffset.x
    }
    
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        
        let index = Int(scrollView.contentOffset.x / scrollView.width)
        let scrollX = scrollView.contentOffset.x - scrollView.width * CGFloat(index)
        let pacentage = scrollX / scrollView.width
        
        // 向后滚动
        if scrollView.contentOffset.x > lastOffsetX {
            
            segmentView.selectViewScroll(pacentage: pacentage, index: index, direction: 1)
        } else {
            
            segmentView.selectViewScroll(pacentage: 1 - pacentage, index: index + 1, direction: 0)
        }
        
    }
    
    func scrollViewDidEndDecelerating(_ scrollView: UIScrollView) {
        
        print(#function)
        
        let index = (scrollView.contentOffset.x + scrollView.width * 0.5) / scrollView.width
        
        segmentView.scrollToIndex(Int(index))
    }
    
    // 设置setContentOffset执行
    func scrollViewDidEndScrollingAnimation(_ scrollView: UIScrollView) {
        
        print(#function)
    }
}

// #MARK - segmentViewDelegate
extension KJSegmentController :  KJSegmentViewDelegate {
    
    func selectIndex(_ index: Int) {
        
        print("点击\(index)")
        
        // 滚动到置顶界面
        scrollView.contentOffset = CGPoint(x: CGFloat(index) * scrollView.width, y: 0)
    }

}
