//
//  KJSegmentView.swift
//  12-swift分段选择
//
//  Created by apple on 2017/9/7.
//  Copyright © 2017年 HEJJY. All rights reserved.
//

import UIKit

let KMargin : CGFloat = 20

protocol KJSegmentViewDelegate : class {
    
    func selectIndex(_ index : Int)
}

class KJSegmentView: UIView {
    
    fileprivate let kNormalColor : (CGFloat, CGFloat, CGFloat) = (0, 0, 0)
    fileprivate let kSelectColor : (CGFloat, CGFloat, CGFloat) = (244, 122, 0)
    
    lazy var buttonArray : [UIButton] = [UIButton]()
    
    weak var delegate : KJSegmentViewDelegate?

    var currentBtn : UIButton = UIButton()
    var lastBtn : UIButton = UIButton()
    
    // 重写set方法添加按钮
    var titleArray : [String] = [String]()  {
        
        didSet {
        
            for i in 0..<titleArray.count {
                
                let button = UIButton()
                button.tag = i
                button.setTitle(titleArray[i], for: .normal)
                button.setTitleColor(UIColor.black, for: .normal)
                button.titleLabel?.font = UIFont.systemFont(ofSize: 17)
                button.addTarget(self, action: #selector(titleBtnClick(_ : )), for: .touchUpInside)
                
                button.sizeToFit()
                button.centerY = self.height * 0.5
                button.x = currentBtn.frame.maxX + KMargin
                scrollView.addSubview(button)
                buttonArray.append(button)
                currentBtn = button
                
                // 默认选中
                if i == 0 {
                    
                    button.setTitleColor(UIColor.orange, for: .normal)
                    selectView.width = currentBtn.width
                    selectView.centerX = currentBtn.centerX
                    lastBtn = button
                }
                
            }
            
            // 设置scrollview的滚动范围
            scrollView.contentSize = CGSize(width: currentBtn.frame.maxX + KMargin, height: self.height)

        }
    }
    
    fileprivate lazy var lineView : UIView = {
       
        let line = UIView(frame: CGRect(x: 0, y: self.height - 0.5, width: self.width, height: 0.5))
        line.backgroundColor = UIColor.lightGray
        return line
        
    }()
    
    fileprivate lazy var selectView : UIView = {
        
        let line = UIView(frame: CGRect(x: 0, y: self.height - 3, width: 30, height: 2))
        line.backgroundColor = UIColor.orange
        return line
    }()
    
    fileprivate lazy var scrollView : UIScrollView = {
    
        let scroll = UIScrollView(frame: self.bounds)
        scroll.showsHorizontalScrollIndicator = false
        scroll.bounces = false
        return scroll
    }()
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        addSubview(scrollView)
        addSubview(lineView)
        scrollView.addSubview(selectView)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /*
    // Only override draw() if you perform custom drawing.
    // An empty implementation adversely affects performance during animation.
    override func draw(_ rect: CGRect) {
        // Drawing code
    }
    */

}

extension KJSegmentView {
    
    func titleBtnClick(_ btn : UIButton)  {
        
        if btn .isEqual(lastBtn) {
            return
        }
        
        lastBtn.setTitleColor(UIColor.black, for: .normal)
        btn.setTitleColor(UIColor.orange, for: .normal)
        
        lastBtn = btn
        
        UIView.animate(withDuration: 0.25) { 
            self.selectView.width = btn.width
            self.selectView.centerX = btn.centerX
        }
        
        let offsetX = btn.centerX - self.width * 0.5
        
        if offsetX >= 0 {
            
            if btn.centerX + self.width * 0.5 >= self.scrollView.contentSize.width {
                
                scrollView.setContentOffset(CGPoint(x: scrollView.contentSize.width - self.width, y: 0), animated: true)
                
            } else {
                scrollView.setContentOffset(CGPoint(x: offsetX, y: 0), animated: true)
            }
            
        } else {
            
            scrollView.setContentOffset(CGPoint(x: 0, y: 0), animated: true)
        }

        delegate?.selectIndex(btn.tag)
    }
    
    // 滚动内容设置标题
    func scrollToIndex(_ index : Int)  {
        
        let button = buttonArray[index]
        
        if button .isEqual(lastBtn) {
            return
        }
        
        lastBtn.setTitleColor(UIColor.black, for: .normal)
        button.setTitleColor(UIColor.orange, for: .normal)
        lastBtn = button
        
        UIView.animate(withDuration: 0.25) {
            self.selectView.width = button.width
            self.selectView.centerX = button.centerX
        }
        
        let offsetX = button.centerX - self.width * 0.5
        
        if offsetX >= 0 {
            
            if button.centerX + self.width * 0.5 >= self.scrollView.contentSize.width {
                
                scrollView.setContentOffset(CGPoint(x: scrollView.contentSize.width - self.width, y: 0), animated: true)

            } else {
                scrollView.setContentOffset(CGPoint(x: offsetX, y: 0), animated: true)
            }
            
        } else {
            
            scrollView.setContentOffset(CGPoint(x: 0, y: 0), animated: true)
        }
        
    }
    
    //        [self.segmentView selectViewScroll:pacentage index:index direction:1];
    
    func selectViewScroll(pacentage : CGFloat, index : Int, direction : Int) {
        
        if (pacentage == 0) {
            return
        }
        
        let colorDelta = (kSelectColor.0 - kNormalColor.0, kSelectColor.1 - kNormalColor.1, kSelectColor.2 - kNormalColor.2)
       
        // 向后移动
        if direction == 1 {
            
            let current = buttonArray[index]
            let next = buttonArray[index + 1]
            
            let distance = next.centerX - current.centerX
            
            // 设置位置
            selectView.width = (next.width - current.width) * pacentage + current.width;
            selectView.centerX = current.centerX + distance * pacentage;
            
            // 用元组设置渐变颜色
            let currentColor = UIColor(r: kSelectColor.0 - colorDelta.0 * pacentage, g: kSelectColor.1 - colorDelta.1 * pacentage, b: kSelectColor.2 - colorDelta.2 * pacentage)
            
            let nextColor = UIColor(r: kNormalColor.0 + colorDelta.0 * pacentage, g: kNormalColor.1 + colorDelta.1 * pacentage, b: kNormalColor.2 + colorDelta.2 * pacentage)
            
            current.setTitleColor(currentColor, for: .normal)
            next.setTitleColor(nextColor, for: .normal)
            
            
        } else {
            
            let current = self.buttonArray[index];
            let before = self.buttonArray[index - 1];
            
            let distance = current.centerX - before.centerX;
            
            // 设置位置
            self.selectView.width = (before.width - current.width) * pacentage + current.width;
            self.selectView.centerX = current.centerX - distance * pacentage;
            
            // 用元组设置渐变颜色
            let currentColor = UIColor(r: kSelectColor.0 - colorDelta.0 * pacentage, g: kSelectColor.1 - colorDelta.1 * pacentage, b: kSelectColor.2 - colorDelta.2 * pacentage)
            
            let beforeColor = UIColor(r: kNormalColor.0 + colorDelta.0 * pacentage, g: kNormalColor.1 + colorDelta.1 * pacentage, b: kNormalColor.2 + colorDelta.2 * pacentage)
            current.setTitleColor(currentColor, for: .normal)
            before.setTitleColor(beforeColor, for: .normal)
        }
        
    }
    
    
}
